@@ -1,4 +1,5 @@
-#!/bin/bash -eu
+#!/bin/bash
+set -eu
 
 ##########################################################################
 ####           (c) Copyright 2004-2007 Canonical Ltd.                #####
@@ -23,7 +24,9 @@
 # Depends: debootstrap, rsync, python-minimal|python, procps, squashfs-tools
 
 cleanup() {
-    for mnt in $MOUNTS ${ROOT}lib/modules/*/volatile ${ROOT}var/{lock,run}; do
+    for mnt in ${ROOT}dev/pts ${ROOT}dev/shm ${ROOT}.dev ${ROOT}dev \
+	       ${ROOT}proc/sys/fs/binfmt_misc ${ROOT}proc ${ROOT}sys \
+	       ${ROOT}lib/modules/*/volatile ${ROOT}var/{lock,run}; do
 	umount $mnt || true
     done
 
@@ -65,34 +68,34 @@ export DEBIAN_FRONTEND=noninteractive
 export LANG=C
 export CASPER_GENERATE_UUID=1
 SRCMIRROR=http://archive.ubuntu.com/ubuntu
-ARCH=$(dpkg --print-installation-architecture)
+PPAMIRROR=ppa.launchpad.net
+ARCH=$(dpkg --print-architecture)
 OPTMIRROR=
+INITRD_COMPRESSOR=lzma
 
 select_mirror () {
     case $ARCH in
-	i386|amd64|sparc)
+	i386|amd64)
 	    case $FS in
-		ubuntu-lpia)
+		ubuntu-lpia|ubuntu-mid)
 		    USERMIRROR=http://ports.ubuntu.com/ubuntu-ports
 		    SECMIRROR=${USERMIRROR}
 		    SECSRCMIRROR=${SRCMIRROR}
+		    TARGETARCH=lpia
 		    ;;
 		*)
 		    USERMIRROR=http://archive.ubuntu.com/ubuntu
 		    SECMIRROR=http://security.ubuntu.com/ubuntu
 		    SECSRCMIRROR=${SECMIRROR}
+		    TARGETARCH=${ARCH}
 		    ;;
 	    esac
 	    ;;
-	hppa)
-	    USERMIRROR=http://ports.ubuntu.com/ubuntu-ports
-	    SECMIRROR=${USERMIRROR}
-	    SECSRCMIRROR=${SRCMIRROR}
-	    ;;
 	*)
 	    USERMIRROR=http://ports.ubuntu.com/ubuntu-ports
 	    SECMIRROR=${USERMIRROR}
 	    SECSRCMIRROR=${SRCMIRROR}
+	    TARGETARCH=${ARCH}
 	    ;;
     esac
     case $(hostname --fqdn) in
@@ -110,12 +113,16 @@ select_mirror () {
     fi
 }
 
-STE=gutsy
+STE=$(lsb_release -cs)
 EXCLUDE=""
 LIST=""
 SUBARCH=""
+PROPOSED=""
+# must be in the "team / PPA name" form; e.g. "moblin/ppa"; the default PPA
+# name is "ppa", don't omit it
+PPA=""
 
-while getopts :d:e:i:I:m:S:s: name; do case $name in
+while getopts :d:e:i:I:m:S:s:a:p name; do case $name in
     d)  STE=$OPTARG;;
     e)  EXCLUDE="$EXCLUDE $OPTARG";;
     i)  LIST="$LIST $OPTARG";;
@@ -123,21 +130,23 @@ while getopts :d:e:i:I:m:S:s: name; do case $name in
     m)	OPTMIRROR="$OPTARG";;
     S)	USZ="$OPTARG";;
     s)	SUBARCH="$OPTARG";;
+    a)	ARCH="$OPTARG";;
+    p)  PROPOSED="yes";;
     \?) echo bad usage >&2; exit 2;;
     \:) echo missing argument >&2; exit 2;;
 esac; done;
 shift $((OPTIND-1))
 
 if (( $# == 0 )) || [ "X$1" = "Xall" ]; then
-    set -- ubuntu kubuntu kubuntu-kde4 edubuntu xubuntu gobuntu base
+    set -- ubuntu kubuntu kubuntu-netbook edubuntu xubuntu mythbuntu gobuntu base
     if [ "$ARCH" = "i386" ]; then
-        set -- ubuntu ubuntu-dvd ubuntu-lpia kubuntu kubuntu-dvd kubuntu-kde4 edubuntu edubuntu-dvd xubuntu gobuntu base
+        set -- ubuntu ubuntu-dvd kubuntu kubuntu-dvd kubuntu-netbook edubuntu edubuntu-dvd mythbuntu xubuntu gobuntu base
     fi
 fi
 
 for arg in "$@"; do
     case "$arg" in
-	ubuntu|ubuntu-dvd|ubuntu-lpia|edubuntu|edubuntu-dvd|kubuntu|kubuntu-dvd|kubuntu-kde4|xubuntu|gobuntu|base|tocd)
+       ubuntu|ubuntu-dvd|ubuntu-lpia|edubuntu|edubuntu-dvd|kubuntu|kubuntu-dvd|kubuntu-netbook|xubuntu|mythbuntu|gobuntu|ubuntu-mid|ubuntu-netbook-remix|ubuntu-moblin-remix|base|tocd)
 	    ;;
 	*)
 	    echo bad name >&2;
@@ -150,12 +159,14 @@ ROOT=$(pwd)/chroot-livecd/	# trailing / is CRITICAL
 for FS in "$@"; do
     FSS="$FS${SUBARCH:+-$SUBARCH}"
     IMG=livecd.${FSS}.fsimg
-    MOUNTS="${ROOT}dev/pts ${ROOT}dev/shm ${ROOT}.dev ${ROOT}dev ${ROOT}proc ${ROOT}sys"
     DEV=""
     COMP="main restricted"
 
     select_mirror
 
+    # Just in case there's some leftover junk here:
+    cleanup 2>/dev/null || true
+
     rm -rf ${ROOT}
 
     mkdir -p ${ROOT}var/cache/debconf
@@ -171,31 +182,50 @@ Flags: seen
     case "$FS" in
 	ubuntu|ubuntu-lpia|ubuntu-dvd)
 	    LIST="$LIST minimal^ standard^ ubuntu-desktop^"
-	    LIVELIST="ubuntu-live^ xresprobe laptop-detect casper lupin-casper"
+	    LIVELIST="ubuntu-live^ laptop-detect casper lupin-casper"
 	    ;;
 	kubuntu|kubuntu-dvd)
 	    LIST="$LIST minimal^ standard^ kubuntu-desktop^"
-	    LIVELIST="kubuntu-live^ xresprobe laptop-detect casper lupin-casper"
+	    LIVELIST="kubuntu-live^ laptop-detect casper lupin-casper"
 	    ;;
-	kubuntu-kde4)
-	    LIST="$LIST minimal^ standard^ kubuntu-kde4-desktop^"
-	    LIVELIST="language-support-en kubuntu-kde4-live^ xresprobe laptop-detect casper lupin-casper"
-	    COMP="main restricted universe multiverse"
+	kubuntu-netbook)
+	    LIST="$LIST minimal^ standard^ kubuntu-netbook^"
+	    LIVELIST="language-support-en kubuntu-netbook-live^ laptop-detect casper lupin-casper"
 	    ;;
 	edubuntu|edubuntu-dvd)
-	    LIST="$LIST minimal^ standard^ edubuntu-desktop^"
-	    LIVELIST="edubuntu-live^ xresprobe laptop-detect casper lupin-casper"
+	    LIST="$LIST minimal^ standard^ edubuntu-desktop-gnome^"
+	    LIVELIST="edubuntu-live^ laptop-detect casper lupin-casper"
 	    ;;
 	xubuntu)
-	    LIST="$LIST minimal^ standard^ xterm libgoffice-gtk-0-6 xubuntu-desktop^"
-	    LIVELIST="xubuntu-live^ xresprobe laptop-detect casper lupin-casper"
+	    LIST="$LIST minimal^ standard^ xterm xubuntu-desktop^"
+	    LIVELIST="xubuntu-live^ laptop-detect casper lupin-casper"
 	    COMP="main restricted universe multiverse"
 	    ;;
 	gobuntu)
 	    LIST="$LIST minimal^ standard^ gobuntu-desktop^"
-	    LIVELIST="gobuntu-live^ xresprobe laptop-detect casper lupin-casper"
+	    LIVELIST="gobuntu-live^ laptop-detect casper lupin-casper"
 	    COMP="main"
 	    ;;
+	ubuntu-mid)
+	    LIST="$LIST minimal^ mobile-mid^"
+	    LIVELIST="mobile-live^ casper ubiquity"
+	    COMP="main restricted universe multiverse"
+	    ;;
+	ubuntu-netbook-remix)
+	    LIST="$LIST minimal^ standard^ ubuntu-netbook-remix^"
+	    LIVELIST="unr-live^ laptop-detect casper lupin-casper"
+	    ;;
+	mythbuntu)
+	    LIST="$LIST minimal^ standard^ mythbuntu-desktop^"
+	    LIVELIST="mythbuntu-live^ laptop-detect casper lupin-casper"
+	    COMP="main restricted universe multiverse"
+	    ;;
+	ubuntu-moblin-remix)
+	    LIST="$LIST minimal^ ubuntu-moblin-remix"
+	    LIVELIST="ubuntu-moblin-live"
+	    COMP="main restricted universe"
+	    PPA="moblin/ppa"
+	    ;;
 	base)
 	    LIST="$LIST minimal^ standard^"
 	    LIVELIST="casper lupin-casper"
@@ -227,22 +257,28 @@ Flags: seen
     case "$FS" in
 	*-dvd)
 	    LIVELIST="$LIVELIST ${FS}-live^"
+	    UNIVERSE=1
+	    MULTIVERSE=1
+	    ;;
+	*)
+	    UNIVERSE=
+	    MULTIVERSE=
 	    ;;
     esac
 
     dpkg -l livecd-rootfs || true	# get our version # in the log.
-    if [ "$FS" != "ubuntu-lpia" ]; then
-        debootstrap --components=$(echo $COMP | sed 's/ /,/g') $STE $ROOT $MIRROR
-    else
-        debootstrap --components=$(echo $COMP | sed 's/ /,/g') --arch lpia $STE $ROOT $MIRROR
-    fi
+    debootstrap --components=$(echo $COMP | sed 's/ /,/g') --arch $TARGETARCH $STE $ROOT $MIRROR
+
+    # Recent dpkg has started complaining pretty loudly if dev/pts isn't 
+    # mounted, so let's get it mounted immediately after debootstrap:
+    mount -t devpts devpts-${STE}-${FSS}-livefs ${ROOT}dev/pts
 
     # Just make a few things go away, which lets us skip a few other things.
     DIVERTS="usr/sbin/mkinitrd usr/sbin/invoke-rc.d"
     for file in $DIVERTS; do
 	mkdir -p ${ROOT}${file%/*}
 	chroot $ROOT dpkg-divert --add --local --divert /${file}.livecd --rename /${file}
-	cp /bin/true ${ROOT}$file
+	cp ${ROOT}/bin/true ${ROOT}$file
     done
 
     # /bin/true won't cut it for mkinitrd, need to have -o support.
@@ -257,8 +293,8 @@ for i in range(len(sys.argv)):
 
     trap "cleanup" 0 1 2 3 15
 
-    case $ARCH in
-        alpha|amd64|i386|ia64|m68k|mips|mipsel)
+    case $TARGETARCH in
+        alpha|amd64|i386|ia64|lpia|m68k|mips|mipsel)
             link_in_boot=no
             ;;
         *)
@@ -279,43 +315,121 @@ link_in_boot = $link_in_boot
     mkdir -p ${ROOT}proc
     mount -tproc none ${ROOT}proc
 
-    case $ARCH+$SUBARCH in
+    case $TARGETARCH+$SUBARCH in
 	powerpc+ps3)
 	    mkdir -p ${ROOT}spu;;
     esac
 
+    case $TARGETARCH in
+	amd64)		LIST="$LIST linux-generic";;
+	i386)		LIST="$LIST linux-generic";;
+
+	# and the bastard stepchildren
+	lpia)		LIST="$LIST linux-lpia";;
+	ia64)		LIST="$LIST linux-ia64";;
+	hppa)		LIST="$LIST linux-hppa32 linux-hppa64";;
+	powerpc)	LIST="$LIST linux-powerpc linux-powerpc64-smp";;
+	sparc*)		LIST="$LIST linux-sparc64";;
+	armel)
+			case "$SUBARCH" in
+				imx51)
+					LIST="$LIST linux-imx51"
+					# temporarily enable universe until kernel is in main
+					;;
+				dove)
+					LIST="$LIST linux-dove"
+					# temporarily enable universe until kernel is in main
+					;;
+			esac;;
+	*)		echo "Unknown architecture: no kernel."; exit 1;;
+    esac
+
+    # this indicates whether or not to keep /boot/vmlinuz; the default is to
+    # strip it from the livefs as ubiquity >= 1.9.4 copies the kernel from the
+    # CD root (/casper/vmlinuz) to the target if it doesn't find one on the
+    # livefs, allowing us to save space; however some subarches use the uImage
+    # format and wouldn't save any space so the stripping doesn't make sense
+    STRIP_VMLINUZ=yes
+    if [ "$TARGETARCH" = "armel" ] && [ "$SUBARCH" = "dove" ]; then
+        STRIP_VMLINUZ=no
+    fi
+
+    for x in $EXCLUDE; do
+	LIST="$(without_package "$x" "$LIST")"
+    done
+
+    if [ "$STE" = "hardy" ]; then
+	# <hack, hack, hack> use the version of ssl-cert from the release
+	# pocket, because the version in -updates pulls in the large
+	# openssl-blacklist package which we should never need on the
+	# live CD
+	cat << @@EOF > ${ROOT}etc/apt/preferences
+Package: ssl-cert
+Pin: version 1.0.14-0ubuntu2
+Pin-Priority: 900
+@@EOF
+    fi
+
+    # Create a good sources.list, and finish the install
+    echo deb $MIRROR $STE ${COMP} > ${ROOT}etc/apt/sources.list
+    echo deb $MIRROR ${STE}-security ${COMP} >> ${ROOT}etc/apt/sources.list
+    echo deb $MIRROR ${STE}-updates ${COMP} >> ${ROOT}etc/apt/sources.list
+    if [ "$PROPOSED" = "yes" ]; then
+        echo deb $MIRROR ${STE}-proposed ${COMP} >> ${ROOT}etc/apt/sources.list
+    fi
+    if [ -n "$PPA" ]; then
+        echo deb http://$PPAMIRROR/$PPA/ubuntu ${STE} main >> ${ROOT}etc/apt/sources.list
+
+        # handle PPAs named "ppa" specially; their Origin field in the Release
+        # file does not end with "-ppa" for backwards compatibility
+        origin="${PPA%/ppa}"
+        origin="${origin/\//-}"
+        touch ${ROOT}etc/apt/preferences
+        cat << @@EOF >> ${ROOT}etc/apt/preferences
+Package: *
+Pin: release o=LP-PPA-$origin
+Pin-Priority: 550
+@@EOF
+    fi
+
+    if [ "$FS" = "ubuntu-moblin-remix" ]; then
+	chroot $ROOT apt-get update
+	chroot $ROOT apt-get -y --force-yes install ubuntu-moblin-ppa-keyring
+	# promote Release.gpg from APT's lists/partial/ to lists/
+	chroot $ROOT apt-get update
+	# workaround LP #442082
+	rm -f ${ROOT}var/cache/apt/{,src}pkgcache.bin
+    fi
+
     # In addition to the ones we got from apt, trust whatever the local system
     # believes in, but put things back afterwards.
     cp ${ROOT}etc/apt/trusted.gpg ${ROOT}etc/apt/trusted.gpg.$$
     cat /etc/apt/trusted.gpg >> ${ROOT}etc/apt/trusted.gpg
 
-    case $ARCH in
-	amd64)		LIST="$LIST linux-generic";;
-	i386)
-	    case $FS in
-		ubuntu-lpia) LIST="$LIST linux-lpia";;
-		*)	LIST="$LIST linux-generic";;
-	    esac;;
-
-	# and the bastard stepchildren
-	ia64)		LIST="$LIST linux-itanium linux-mckinley";;
-	hppa)		LIST="$LIST linux-hppa32 linux-hppa64";;
-	powerpc)	LIST="$LIST linux-powerpc linux-powerpc64-smp";;
-	sparc*)		LIST="$LIST linux-sparc64";;
-	*)		echo "Unknown architecture: no kernel."; exit 1;;
-    esac
-
-    for x in $EXCLUDE; do
-	LIST="$(without_package "$x" "$LIST")"
-    done
-
-    # Create a good sources.list, and finish the install
-    echo deb $MIRROR $STE ${COMP} > ${ROOT}etc/apt/sources.list
     chroot $ROOT apt-get update
-    chroot $ROOT apt-get -y install $LIST </dev/null
+    chroot $ROOT apt-get -y --purge dist-upgrade </dev/null
+    chroot $ROOT apt-get -y --purge install $LIST </dev/null
+
+    # launchpad likes to put dependencies of seeded packages in tasks along with the
+    # actual seeded packages.  In general, this isn't an issue.  With updated kernels
+    # and point-releases, though, we end up with extra header packages:
+    chroot ${ROOT} dpkg -l linux-headers-2\* | grep ^i | awk '{print $2}' \
+        > livecd.${FSS}.manifest-headers
+    chroot ${ROOT} dpkg -l linux-headers-\* | grep ^i | awk '{print $2}' \
+        > livecd.${FSS}.manifest-headers-full
+    HEADERPACKAGES=`cat livecd.${FSS}.manifest-headers-full`
+    HEADERMETA=""
+    for i in `comm -3 livecd.${FSS}.manifest-headers livecd.${FSS}.manifest-headers-full`; do
+        HEADERMETA="$HEADERMETA $i"
+    done
+    rm -f livecd.${FSS}.manifest-headers livecd.${FSS}.manifest-headers-full
+    chroot ${ROOT} apt-get -y --purge remove $HEADERPACKAGES </dev/null || true
+    chroot ${ROOT} apt-get -y --purge install $HEADERMETA </dev/null || true
+    # End horrible linux-header launchpad workaround.  Hopefully this is temporary.
+
     chroot ${ROOT} dpkg-query -W --showformat='${Package} ${Version}\n' \
 	> livecd.${FSS}.manifest-desktop
-    chroot $ROOT apt-get -y install $LIVELIST </dev/null
+    chroot $ROOT apt-get -y --purge install $LIVELIST </dev/null
     case $FS in
 	edubuntu)
 	    chroot $ROOT apt-cache dumpavail | \
@@ -341,36 +455,94 @@ link_in_boot = $link_in_boot
 	chroot $ROOT dpkg-divert --remove --rename /${file}
     done
 
+    # remove the apt preferences hack if it was added
+    rm -f ${ROOT}etc/apt/preferences
+
     # And make this look more pristine
-    cleanup
     cat << @@EOF > ${ROOT}etc/apt/sources.list
 deb ${USERMIRROR} $STE ${COMP}
 deb-src ${SRCMIRROR} $STE ${COMP}
 
+deb ${SECMIRROR} ${STE}-security ${COMP}
+deb-src ${SECSRCMIRROR} ${STE}-security ${COMP}
+
+## Major bug fix updates produced after the final release of the
+## distribution.
+deb ${USERMIRROR} ${STE}-updates ${COMP}
+deb-src ${SRCMIRROR} ${STE}-updates ${COMP}
+
+@@EOF
+    if [ "$UNIVERSE" ]; then
+	COMMENT=
+    else
+	cat << @@EOF >> ${ROOT}etc/apt/sources.list
 ## Uncomment the following two lines to add software from the 'universe'
 ## repository.
+@@EOF
+	COMMENT='# '
+    fi
+    cat << @@EOF >> ${ROOT}etc/apt/sources.list
+## N.B. software from this repository is ENTIRELY UNSUPPORTED by the Ubuntu
+## team. Also, please note that software in universe WILL NOT receive any
+## review or updates from the Ubuntu security team.
+${COMMENT}deb ${USERMIRROR} $STE universe
+${COMMENT}deb-src ${SRCMIRROR} $STE universe
+${COMMENT}deb ${USERMIRROR} ${STE}-updates universe
+${COMMENT}deb-src ${SRCMIRROR} ${STE}-updates universe
+${COMMENT}deb ${SECMIRROR} ${STE}-security universe
+${COMMENT}deb-src ${SECSRCMIRROR} ${STE}-security universe
+
+@@EOF
+    if [ "$MULTIVERSE" ]; then
+	COMMENT=
+    else
+	COMMENT='# '
+    fi
+    cat << @@EOF >> ${ROOT}etc/apt/sources.list
 ## N.B. software from this repository is ENTIRELY UNSUPPORTED by the Ubuntu
 ## team, and may not be under a free licence. Please satisfy yourself as to
 ## your rights to use the software. Also, please note that software in
-## universe WILL NOT receive any review or updates from the Ubuntu security
-## team.
-# deb ${USERMIRROR} $STE universe
-# deb-src ${SRCMIRROR} $STE universe
-
-deb ${SECMIRROR} ${STE}-security ${COMP}
-deb-src ${SECSRCMIRROR} ${STE}-security ${COMP}
+## multiverse WILL NOT receive any review or updates from the Ubuntu
+## security team.
+${COMMENT}deb ${USERMIRROR} $STE multiverse
+${COMMENT}deb-src ${SRCMIRROR} $STE multiverse
+${COMMENT}deb ${USERMIRROR} ${STE}-updates multiverse
+${COMMENT}deb-src ${SRCMIRROR} ${STE}-updates multiverse
+${COMMENT}deb ${SECMIRROR} ${STE}-security multiverse
+${COMMENT}deb-src ${SECSRCMIRROR} ${STE}-security multiverse
 @@EOF
+    if [ -n "$PPA" ]; then
+    cat << @@EOF >> ${ROOT}etc/apt/sources.list
+
+## The following unsupported and untrusted Personal Archives (PPAs) were used
+## to create the base image of this system
+deb http://$PPAMIRROR/$PPA/ubuntu ${STE} main
+deb-src http://$PPAMIRROR/$PPA/ubuntu ${STE} main
+@@EOF
+
+        # handle PPAs named "ppa" specially; their Origin field in the Release
+        # file does not end with "-ppa" for backwards compatibility
+        origin="${PPA%/ppa}"
+        origin="${origin/\//-}"
+        touch ${ROOT}etc/apt/preferences
+        cat << @@EOF >> ${ROOT}etc/apt/preferences
+Explanation: This prefers the Personal Archive $PPA over the other sources
+Package: *
+Pin: release o=LP-PPA-$origin
+Pin-Priority: 550
+@@EOF
+    fi
     mv ${ROOT}etc/apt/trusted.gpg.$$ ${ROOT}etc/apt/trusted.gpg
 
     # get rid of the .debs - we don't need them.
     chroot ${ROOT} apt-get clean
-    rm -f ${ROOT}etc/X11/xorg.conf
-    # Restore an empty xorg.conf, else xserver-xorg postinst will be confused
-    touch ${ROOT}etc/X11/xorg.conf
     rm -f ${ROOT}var/lib/apt/lists/*_*
     rm -f ${ROOT}var/spool/postfix/maildrop/*
     # Removing update-notifier notes is now considered harmful:
     #rm -f ${ROOT}var/lib/update-notifier/user.d/*
+    # The D-Bus machine identifier needs to be unique, and is generated at
+    # boot time if it's missing.
+    rm -f ${ROOT}var/lib/dbus/machine-id
     chroot $ROOT apt-get update || true	# give them fresh lists, but don't fail
     rm -f ${ROOT}etc/resolv.conf ${ROOT}etc/mailname
     if [ -f ${ROOT}/etc/postfix/main.cf ]; then
@@ -384,16 +556,28 @@ deb-src ${SECSRCMIRROR} ${STE}-security ${COMP}
 	chroot ${ROOT} update-initramfs -k "${KVER}" -u
 	# we mv the initramfs, so it's not wasting space on the livefs
 	mv ${ROOT}/boot/initrd.img-"${KVER}" livecd.${FSS}.initrd-"${SUBARCH}"
-	cp ${ROOT}/boot/vmlinu?-"${KVER}" livecd.${FSS}.kernel-"${SUBARCH}"
+	rm -f ${ROOT}/boot/initrd.img-"${KVER}".bak
+	# whether to strip vmlinuz or not to save space thanks to ubiquity
+	action="cp"
+	if [ "$STRIP_VMLINUZ" = "yes" ]; then
+	    action="mv"
+	fi
+	$action ${ROOT}/boot/vmlinu?-"${KVER}" livecd.${FSS}.kernel-"${SUBARCH}"
+	if [ "$INITRD_COMPRESSOR" != gz ]; then
+	    zcat "livecd.${FSS}.initrd-${SUBARCH}" | "$INITRD_COMPRESSOR" -9c \
+		> "livecd.${FSS}.initrd-${SUBARCH}.new"
+	    mv "livecd.${FSS}.initrd-${SUBARCH}.new" \
+		"livecd.${FSS}.initrd-${SUBARCH}"
+	fi
     done
     NUMKVERS="$(set -- $KVERS; echo $#)"
     if [ "$NUMKVERS" = 1 ]; then
 	# only one kernel
 	SUBARCH="${KVERS#*-*-}"
-	ln -s livecd.${FSS}.initrd-"${SUBARCH}" livecd.${FSS}.initrd
-	ln -s livecd.${FSS}.kernel-"${SUBARCH}" livecd.${FSS}.kernel
+	ln -sf livecd.${FSS}.initrd-"${SUBARCH}" livecd.${FSS}.initrd
+	ln -sf livecd.${FSS}.kernel-"${SUBARCH}" livecd.${FSS}.kernel
     fi
-    case $ARCH+$SUBARCH in
+    case $TARGETARCH+$SUBARCH in
 	powerpc+ps3)
 	    chroot ${ROOT} addgroup --system spu;;
     esac
@@ -426,17 +610,22 @@ deb-src ${SECSRCMIRROR} ${STE}-security ${COMP}
     perl -i -nle 'print unless /^Package: language-(pack|support)/ .. /^$/;' \
         ${ROOT}/var/lib/apt/extended_states
 
+  # And run the cleanup function dead last, to umount /proc after nothing
+  # else needs to be run in the chroot (umounting it earlier breaks rm):
+  cleanup
+
   livefs_squash()
   {
-    squashsort="http://people.ubuntu.com/~tfheen/livesort/${FSS}.list.${ARCH}"
-    if wget -O livecd.${FSS}.sort ${squashsort} > /dev/null 2>&1; then
+    squashsort="http://people.ubuntu.com/~tfheen/livesort/${FSS}.list.${TARGETARCH}"
+    #if wget -O livecd.${FSS}.sort ${squashsort} > /dev/null 2>&1; then
+    if false; then
       echo "Using the squashfs sort list from ${squashsort}."
     else
       echo "Unable to fetch squashfs sort list; using a blank list."
       : > livecd.${FSS}.sort
     fi
 
-    mksquashfs ${ROOT} livecd.${FSS}.squashfs -nolzma -sort livecd.${FSS}.sort
+    mksquashfs ${ROOT} livecd.${FSS}.squashfs -sort livecd.${FSS}.sort
     chmod 644 livecd.${FSS}.squashfs
   }
 
